/*!
 * touch-lite: A lightweight touch library without any dependency.
 * Version 0.0.6
 * Copyright (C) 2017 JounQin <admin@1stg.me>
 * Released under the MIT license
 *
 * Github: https://github.com/JounQin/touch-lite
 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define('touch-lite', factory) :
	(global.Touch = factory());
}(this, (function () { 'use strict';

var trueType = function (value) { return Object.prototype.toString.call(value).split('').slice(8, -1).join(''); };

var trueTypeFunc = function (type) { return function (value) { return type === trueType(value); }; };

var utils = {};

var TYPES = ['Array', 'Function', 'Object', 'String'];

for (var i = 0, len = TYPES.length; i < len; i++) {
  var type = TYPES[i];
  utils[("is" + type)] = trueTypeFunc(type);
}

var isArray = utils.isArray;
var isFunction = utils.isFunction;
var isObject = utils.isObject;
var isString = utils.isString;

var on = function (el, type, listener) {
  if (isObject(type)) {
    for (var t in type) {
      on(el, t, type[t]);
    }
    return
  }

  if (!isString(type) || /^ +$/.test(type)) { return }

  var events = type.split(' ');

  var len;

  if ((len = events.length) > 1) {
    for (var i = 0; i < len; i++) {
      on(el, events[i], listener);
    }
    return
  }

  if (el.addEventListener) {
    el.addEventListener(type, listener, false);
  } else if (el.attachEvent) {
    el.attachEvent('on' + type, listener);
  } else {
    el['on' + type] = listener;
  }
};

var off = function (el, type, listener) {
  if (isObject(type)) {
    for (var t in type) {
      off(el, t, type[t]);
    }
    return
  }

  if (el.removeEventListener) {
    el.removeEventListener(type, listener, false);
  } else if (el.detachEvent) {
    el.detachEvent('on' + type, listener);
  } else {
    el['on' + type] = null;
  }
};

var extend = function (target) {
  var sources = [], len$1 = arguments.length - 1;
  while ( len$1-- > 0 ) sources[ len$1 ] = arguments[ len$1 + 1 ];

  for (var i = 0, len = sources.length; i < len; i++) {
    var source = sources[i];
    if (source != null) {
      for (var key in source) {
        target[key] = source[key];
      }
    }
  }
  return target
};

var indexOf = function (arr, item) {
  for (var i = 0, len = arr.length; i < len; i++) {
    if (arr[i] === item) { return i }
  }
  return -1
};

var MOUSE_DOWN = 'mousedown';
var MOUSE_MOVE = 'mousemove';
var MOUSE_UP = 'mouseup';

var EVENT = {
  start: ("touchstart " + MOUSE_DOWN),
  move: 'touchmove',
  end: 'touchend touchcancel'
};

var Touch = function Touch(el, ref) {
  if ( ref === void 0 ) ref = {};
  var events = ref.events;

  this.el = el;
  this.touchSupport = true;
  this._listeners = {};

  this._init();

  if (!events) { return }

  this.on(events);
};

Touch.prototype.on = function on (event, handler) {
    var this$1 = this;

  if (isObject(event)) {
    for (var e in event) {
      this$1.on(e, event[e]);
    }
    return this
  }

  if (isArray(handler)) {
    for (var i = 0, len = handler.length; i < len; i++) {
      this$1.on(event, handler[i]);
    }
    return this
  }

  if (!isFunction(handler)) { return this }

  var handlers = this._listeners[event];

  if (!handlers) {
    handlers = this._listeners[event] = [];
  }

  if (indexOf(handlers, handler) === -1) {
    handlers.push(handler);
  }

  return this
};

Touch.prototype.off = function off (event, handler) {
  if (!handler) {
    this._listeners[event] = [];
    return this
  }

  var handlers = this._listeners[event];

  var index = indexOf(handlers, handler);

  if (index !== -1) {
    handlers.splice(index, 1);
  }

  return this
};

Touch.prototype.trigger = function trigger (event, e) {
    var this$1 = this;

  var handlers = this._listeners[event];

  if (!handlers) { return }

  for (var i = 0, len = handlers.length; i < len; i++) {
    if (handlers[i].call(this$1, e) === false) {
      return false
    }
  }
};

Touch.prototype.destroy = function destroy () {
  off(this.el, ( obj = {}, obj[EVENT.start] = this._eStart, obj[EVENT.move] = this._eMove, obj[EVENT.end] = this._eEnd, obj ));
    var obj;
};

Touch.prototype._init = function _init () {
  this._start();
  this._move();
  this._end();
};

Touch.prototype.actualEvent = function actualEvent (e, prevent, stop) {
  prevent && e.preventDefault && e.preventDefault();
  stop && e.stopPropagation && e.stopPropagation();
  var touches = e.changedTouches;
  return touches ? touches[0] : e
};

Touch.prototype._isPrevent = function _isPrevent (event, e) {
  if (this.trigger(event, e) === false) { return true }
};

Touch.prototype._start = function _start () {
    var this$1 = this;

  clearTimeout(this._timeout);

  on(
    this.el,
    EVENT.start,
    (this._eStart = function (e) {
      var isMouseDown = e.type === MOUSE_DOWN;

      if (isMouseDown) {
        this$1.touchSupport = false;
        on(document, ( obj = {}, obj[MOUSE_MOVE] = this$1._eMove, obj[MOUSE_UP] = this$1._eEnd, obj ));
          var obj;
      }

      e = this$1.actualEvent(e, !isMouseDown && /Android 4\./.test(navigator.userAgent));

      extend(this$1, {
        _clientX: e.clientX,
        _clientY: e.clientY,
        _startTime: Date.now()
      });

      this$1._isPrevent('start', e) && (this$1._doNotMove = true);
    })
  );
};

Touch.prototype._move = function _move () {
    var this$1 = this;

  on(
    this.el,
    EVENT.move,
    (this._eMove = function (e) {
      if (!this$1._startTime || this$1._doNotMove) { return }

      e = this$1.actualEvent(e);

      var clientX = e.clientX;
        var clientY = e.clientY;
      var ref = this$1;
        var originalClientX = ref._clientX;
        var originalClientY = ref._clientY;

      var changedX = clientX - originalClientX;
      var changedY = clientY - originalClientY;

      if (Math.abs(changedX) > 5 || Math.abs(changedY) > 5) { this$1._moved = true; }

      if (this$1._moved && !this$1._moveStarted) {
        if (this$1._isPrevent('moveStart', e)) { return }
        this$1._moveStarted = true;
      }

      if (!this$1._moveStarted) { return }

      extend(this$1, { changedX: changedX, changedY: changedY });

      this$1._isPrevent('moving', e);
    })
  );
};

Touch.prototype._end = function _end () {
    var this$1 = this;

  on(
    this.el,
    EVENT.end,
    (this._eEnd = function (e) {
      if (e.type === MOUSE_UP) {
        off(document, ( obj = {}, obj[MOUSE_MOVE] = this$1._eMove, obj[MOUSE_UP] = this$1._eEnd, obj ));
          var obj;
      }

      if (!this$1._startTime) { return }

      var ref = this$1;
        var _clientX = ref._clientX;
        var _clientY = ref._clientY;
        var _moved = ref._moved;
        var _startTime = ref._startTime;

      delete this$1._clientX;
      delete this$1._clientY;
      delete this$1._doNotMove;
      delete this$1._moved;
      delete this$1._moveStarted;
      delete this$1._startTime;

      if (_moved) {
        var changedX = e.clientX - _clientX;
        var changedY = e.clientY - _clientY;

        extend(this$1, { changedX: changedX, changedY: changedY });

        if (this$1._isPrevent('moveEnd', e)) { return }

        var absChangedX = Math.abs(changedX);
        var absChangedY = Math.abs(changedY);

        var event;
        if (absChangedX < 20) {
          if (changedY > 50) {
            event = 'swipeDown';
          } else if (changedY < -50) {
            event = 'swipeUp';
          }
        } else if (absChangedY < 20) {
          if (changedX > 50) {
            event = 'swipeRight';
          } else if (changedX < -50) {
            event = 'swipeLeft';
          }
        }
        if (this$1._isPrevent(event, e)) { return }

        return this$1._isPrevent('end', e)
      }

      var duration = Date.now() - _startTime;

      this$1._tapped = this$1._tapped + 1 || 1;

      if (duration > 200) { return this$1._isPrevent('press', e) && this$1._isPrevent('end', e) }

      this$1._timeout = setTimeout(function () {
        var tapped = this$1._tapped;

        delete this$1._tapped;
        delete this$1._timeout;

        if (tapped < 3) {
          var isSingle = tapped === 1;
          var tapEvent = isSingle ? 'tap' : 'dblTap';

          if (this$1._isPrevent(tapEvent, e)) { return }

          var eventInit = {
            bubbles: true,
            cancelable: true,
            cancelBubble: true
          };

          var prefix = isSingle ? '' : 'dbl';

          if (this$1.touchSupport && e.target.dispatchEvent(new Event((prefix + "click"), eventInit)) === false) { return }

          if (e.target.dispatchEvent(new Event((prefix + "tap"), eventInit)) === false) { return }
        } else {
          extend(this$1, { tapped: tapped });
          if (this$1._isPrevent('mltTap', e)) { return }
        }
        this$1._isPrevent('end', e);
      }, 200);
    })
  );
};

return Touch;

})));
//# sourceMappingURL=touch.js.map
